@@ -18,7 +18,8 @@ import flax
 from flax import linen as nn
 from jax import lax
 from jax import numpy as jnp
-from netket.nn.initializers import lecun_normal, zeros
+from jax.nn.initializers import lecun_normal, zeros
+
 from netket.utils.types import Array, DType, NNInitFunc
 
 default_kernel_init = lecun_normal()
