@@ -20,9 +20,9 @@ from jax import lax
 import jax.numpy as jnp
 import numpy as np
 import jax
-from jax.nn.initializers import normal, zeros, lecun_normal, variance_scaling
+from jax.nn.initializers import zeros, lecun_normal
 
-from netket.nn.initializers import _complex_truncated_normal
+from netket.utils import warn_deprecation
 from netket.utils import HashableArray
 from netket.utils.types import Array, DType, PRNGKeyT, Shape, NNInitFunc
 from netket.utils.group import PermutationGroup
@@ -30,75 +30,27 @@ from typing import Sequence
 from netket.graph import Graph, Lattice
 import warnings
 
-
-class _default_densesymm_initializer:
-    pass
+# All layers defined here have kernels of shape [out_features, in_features, n_symm]
+default_equivariant_initializer = lecun_normal(in_axis=1, out_axis=0)
 
 
-default_densesymm_initializer = _default_densesymm_initializer()
-"""
-Sentinel value to signal the default densesymm initializer.
-"""
+def _normalise_mask(mask, new_norm):
+    mask = jnp.asarray(mask)
+    return mask / jnp.linalg.norm(mask) * new_norm ** 0.5
 
 
-def get_default_densesymm_init(real_input_size, *, scale=1):
-    """
-    Returns a kernel init equivalent to lecun_normal but
-    whose scale depends on the number of physical degrees of freedom of the
-    layer.
-
-    This is the correct initialization to have in order to guarantee variance
-    1 of every output channel.
-    """
-
-    def init(key, shape, dtype):
-        shape = jax.core.as_named_shape(shape)
-
-        variance = jnp.array(scale / real_input_size, dtype=dtype)
-
-        if jnp.issubdtype(dtype, jnp.floating):
-            # constant is stddev of standard normal truncated to (-2, 2)
-            stddev = jnp.sqrt(variance) / 0.87962566103423978
-            return jax.random.truncated_normal(key, -2, 2, shape, dtype) * stddev
-        else:
-            stddev = jnp.sqrt(variance) / 0.95311164380491208
-            return _complex_truncated_normal(key, 2, shape, dtype) * stddev
-
-    return init
-
-
-def unit_normal_scaling(key, shape, dtype):
-    return jax.random.normal(key, shape, dtype) / jnp.sqrt(
-        jnp.prod(jnp.asarray(shape[1:]))
+def symm_input_warning(x_shape, new_x_shape, name):
+    warn_deprecation(
+        (
+            f"{len(x_shape)}-dimensional input to {name} layer is deprecated.\n"
+            f"Input shape {x_shape} has been reshaped to {new_x_shape}, where "
+            "the middle dimension encodes different input channels.\n"
+            "Please provide a 3-dimensional input.\nThis warning will become an "
+            "error in the future."
+        )
     )
 
 
-def _symmetrizer_col(perms, features):
-    r"""
-    Creates the mapping from symmetry-reduced kernel w to full kernel W, s.t.
-        W[ij] = S[ij][kl] w[kl]
-    where [ij] ∈ [0,...,n_sites×n_hidden) and [kl] ∈ [0,...,n_sites×features).
-    For each [ij] there is only one [kl] such that S[ij][kl] is non-zero, in which
-    case S[ij][kl] == 1. Thus, this method only returns the array of indices `col`
-    of shape (n_sites×n_hidden,) satisfying
-        W[ij] = w[col[ij]]  <=>  W = w[col].
-
-    See test/models/test_nn.py:test_symmetrizer for how this relates to the
-    matrix form of the symmetrizer.
-    """
-    n_symm, n_sites = perms.shape
-    n_hidden = features * n_symm
-
-    ij = np.arange(n_sites * n_hidden)
-    i, j = np.unravel_index(ij, (n_sites, n_hidden))
-
-    k = perms[j % n_symm, i]
-    l = np.floor_divide(j, n_symm)
-    kl = np.ravel_multi_index((k, l), (n_sites, features))
-
-    return kl
-
-
 class DenseSymmMatrix(Module):
     r"""Implements a symmetrized linear transformation over a permutation group
     using matrix multiplication."""
@@ -108,7 +60,7 @@ class DenseSymmMatrix(Module):
         Numpy/Jax arrays must be wrapped into an :class:`netket.utils.HashableArray`.
     """
     features: int
-    """The number of symmetry-reduced features. The full output size is len(symmetries) * features."""
+    """The number of output features. Will be the second dimension of the output."""
     use_bias: bool = True
     """Whether to add a bias to the output (default: True)."""
     mask: Optional[HashableArray] = None
@@ -118,34 +70,16 @@ class DenseSymmMatrix(Module):
     precision: Any = None
     """numerical precision of the computation see `jax.lax.Precision`for details."""
 
-    kernel_init: NNInitFunc = default_densesymm_initializer
-    """Initializer for the Dense layer matrix. Defaults to variance scaling"""
+    kernel_init: NNInitFunc = default_equivariant_initializer
+    """Initializer for the kernel. Defaults to Lecun normal."""
     bias_init: NNInitFunc = zeros
-    """Initializer for the bias. Defaults to zero initialization"""
+    """Initializer for the bias. Defaults to zero initialization."""
 
     def setup(self):
         # pylint: disable=attribute-defined-outside-init
-        perms = np.asarray(self.symmetries)
-        self.n_symm, self.n_sites = perms.shape
-        self.n_hidden = self.features * self.n_symm
-
-        self.symm_cols = jnp.asarray(_symmetrizer_col(perms, self.features))
-
-    def full_kernel(self, kernel):
-        """
-        Converts the symmetry-reduced kernel of shape (n_sites, features) to
-        the full Dense kernel of shape (n_sites, features * n_symm).
-        """
-        kernel = kernel.transpose(1, 0).reshape(-1)
-        result = kernel[self.symm_cols]
-        return result.reshape(self.n_sites, -1)
-
-    def full_bias(self, bias):
-        """
-        Convert symmetry-reduced bias of shape (features,) to the full bias of
-        shape (n_symm * features,).
-        """
-        return jnp.expand_dims(bias, (0, 2))
+        self.n_symm, self.n_sites = np.asarray(self.symmetries).shape
+        if self.mask is not None:
+            self.scaled_mask = _normalise_mask(self.mask, self.n_sites)
 
     @compact
     def __call__(self, x: Array) -> Array:
@@ -159,35 +93,52 @@ class DenseSymmMatrix(Module):
         """
         dtype = jnp.promote_types(x.dtype, self.dtype)
         x = jnp.asarray(x, dtype)
+        # infer in_features and ensure input dimensions (batch, in_features,n_sites)
+
+        # TODO: Deprecated: Eventually remove and error if less than 3 dimensions
+        if x.ndim < 3:
+            old_shape = x.shape
+            if x.ndim == 1:
+                x = jnp.expand_dims(x, (0, 1))
+            elif x.ndim == 2:
+                x = jnp.expand_dims(x, 1)
+            symm_input_warning(old_shape, x.shape, "DenseSymm")
+
+        in_features = x.shape[1]
 
-        # generate the default kernel init if necessary
-        kernel_init = (
-            get_default_densesymm_init(x.shape[-1])
-            if self.kernel_init is default_densesymm_initializer
-            else self.kernel_init
-        )
         kernel = self.param(
-            "kernel", kernel_init, (self.features, self.n_sites), self.dtype
+            "kernel",
+            self.kernel_init,
+            (self.features, in_features, self.n_sites),
+            self.dtype,
         )
 
         if self.mask is not None:
-            kernel = kernel * jnp.expand_dims(self.mask, 0)
+            kernel = kernel * jnp.expand_dims(self.scaled_mask, (0, 1))
 
-        kernel = self.full_kernel(kernel).reshape(-1, self.features, self.n_symm)
+        # Converts the convolutional kernel of shape (self.features, in_features, n_sites)
+        # to a full dense kernel of shape (self.features, in_features, n_symm, n_sites).
+        # result[out, in, g, r] == kernel[out, in, g^{-1}r]
+        kernel = jnp.take(kernel, jnp.asarray(self.symmetries), 2)
         kernel = jnp.asarray(kernel, dtype)
 
+        # x is      (batches,       in_featuers,         n_sites)
+        # kernel is (self.features, in_features, n_symm, n_sites)
         x = lax.dot_general(
             x,
             kernel,
-            (((x.ndim - 1,), (0,)), ((), ())),
+            (((x.ndim - 2, x.ndim - 1), (1, 3)), ((), ())),
             precision=self.precision,
         )
 
-        x = x.reshape(-1, self.features, self.n_symm)
-
         if self.use_bias:
             bias = self.param("bias", self.bias_init, (self.features,), self.dtype)
-            bias = jnp.asarray(self.full_bias(bias), dtype)
+
+            # Convert symmetry-reduced bias of shape (features,) to the full bias of
+            # shape (..., features, 1).
+            bias = jnp.expand_dims(bias, 1)
+            bias = jnp.asarray(bias, dtype)
+
             x += bias
 
         return x
@@ -199,7 +150,7 @@ class DenseSymmFFT(Module):
     space_group: HashableArray
     """Array that lists the space group as permutations"""
     features: int
-    """The number of input features; must be the second dimension of the input."""
+    """The number of output features. Will be the second dimension of the output."""
     shape: Tuple
     """Tuple that corresponds to shape of lattice"""
     use_bias: bool = True
@@ -210,30 +161,30 @@ class DenseSymmFFT(Module):
     """The dtype of the weights."""
     precision: Any = None
 
-    kernel_init: NNInitFunc = lecun_normal()
-    """Initializer for the Dense layer matrix. Defaults to variance scaling"""
+    kernel_init: NNInitFunc = default_equivariant_initializer
+    """Initializer for the kernel. Defaults to Lecun normal."""
     bias_init: NNInitFunc = zeros
-    """Initializer for the bias. Defaults to zero initialization"""
+    """Initializer for the bias. Defaults to zero initialization."""
 
     def setup(self):
         sg = np.asarray(self.space_group)
 
         self.n_cells = np.product(np.asarray(self.shape))
-        self.n_symm = len(sg) // self.n_cells
+        self.n_point = len(sg) // self.n_cells
         self.sites_per_cell = sg.shape[1] // self.n_cells
 
+        if self.mask is not None:
+            self.scaled_mask = _normalise_mask(self.mask, sg.shape[1])
+
+        # maps (n_sites) dimension of kernels to (sites_per_cell, n_point, *shape)
+        # as used in FFT-based group convolution
         self.mapping = (
             sg[:, : self.sites_per_cell]
-            .reshape(self.n_cells, self.n_symm, self.sites_per_cell)
-            .transpose(1, 2, 0)
-            .reshape(self.n_symm, self.sites_per_cell, *self.shape)
+            .reshape(self.n_cells, self.n_point, self.sites_per_cell)
+            .transpose(2, 1, 0)
+            .reshape(self.sites_per_cell, self.n_point, *self.shape)
         )
 
-    def make_kernel(self, kernel):
-        kernel = kernel[..., self.mapping]
-
-        return kernel
-
     @compact
     def __call__(self, x: Array) -> Array:
         """Applies the equivariant transform to the inputs along the last two
@@ -243,40 +194,49 @@ class DenseSymmFFT(Module):
         dtype = jnp.promote_types(x.dtype, self.dtype)
         x = jnp.asarray(x, dtype)
 
-        x = (
-            x.reshape(-1, self.n_cells, self.sites_per_cell)
-            .transpose(0, 2, 1)
-            .reshape(-1, self.sites_per_cell, *self.shape)
-        )
+        # TODO: Deprecated: Eventually remove and error if less than 3 dimensions
+        # infer in_features and ensure input dimensions (batch, in_features,n_sites)
+        if x.ndim < 3:
+            old_shape = x.shape
+            if x.ndim == 1:
+                x = jnp.expand_dims(x, (0, 1))
+            elif x.ndim == 2:
+                x = jnp.expand_dims(x, 1)
+            symm_input_warning(old_shape, x.shape, "DenseSymm")
+
+        in_features = x.shape[1]
+
+        x = x.reshape(*x.shape[:-1], self.n_cells, self.sites_per_cell)
+        x = x.transpose(0, 1, 3, 2)
+        x = x.reshape(*x.shape[:-1], *self.shape)
 
-        # generate the default kernel init if necessary
-        kernel_init = (
-            get_default_densesymm_init(self.n_cells * self.sites_per_cell)
-            if self.kernel_init is default_densesymm_initializer
-            else self.kernel_init
-        )
         kernel = self.param(
             "kernel",
-            kernel_init,
-            (self.features, self.n_cells * self.sites_per_cell),
+            self.kernel_init,
+            (self.features, in_features, self.n_cells * self.sites_per_cell),
             self.dtype,
         )
 
         kernel = jnp.asarray(kernel, dtype)
 
         if self.mask is not None:
-            kernel = kernel * jnp.expand_dims(self.mask, 0)
+            kernel = kernel * jnp.expand_dims(self.scaled_mask, (0, 1))
 
-        kernel = self.make_kernel(kernel)
+        # Converts the convolutional kernel of shape (features, in_features, n_sites)
+        # to the expanded kernel of shape (features, in_features, sites_per_cell,
+        # n_point, *shape) used in FFT-based group convolutions.
+        kernel = kernel[..., self.mapping]
 
-        x = jnp.fft.fftn(x, s=self.shape).reshape(*x.shape[:2], self.n_cells)
+        x = jnp.fft.fftn(x, s=self.shape).reshape(*x.shape[:3], self.n_cells)
 
         kernel = jnp.fft.fftn(kernel, s=self.shape).reshape(
-            *kernel.shape[:3], self.n_cells
+            *kernel.shape[:4], self.n_cells
         )
 
+        # TODO: the batch ordering should be revised: batch dimensions should
+        # be leading
         x = lax.dot_general(
-            x, kernel, (((1,), (2,)), ((2,), (3,))), precision=self.precision
+            x, kernel, (((1, 2), (1, 2)), ((3,), (4,))), precision=self.precision
         )
         x = x.transpose(1, 2, 3, 0)
         x = x.reshape(*x.shape[:3], *self.shape)
@@ -304,10 +264,8 @@ class DenseEquivariantFFT(Module):
 
     product_table: HashableArray
     """ product table for space group"""
-    in_features: int
-    """The number of input features; must be the second dimension of the input."""
-    out_features: int
-    """The number of input features; must be the second dimension of the input."""
+    features: int
+    """The number of output features. Will be the second dimension of the output."""
     shape: Tuple
     """Tuple that corresponds to shape of lattice"""
     use_bias: bool = True
@@ -319,10 +277,10 @@ class DenseEquivariantFFT(Module):
     precision: Any = None
     """numerical precision of the computation see `jax.lax.Precision`for details."""
 
-    kernel_init: NNInitFunc = unit_normal_scaling
-    """Initializer for the Dense layer matrix. Defaults to variance scaling"""
+    kernel_init: NNInitFunc = default_equivariant_initializer
+    """Initializer for the kernel. Defaults to Lecun normal."""
     bias_init: NNInitFunc = zeros
-    """Initializer for the bias. Defaults to zero initialization"""
+    """Initializer for the bias. Defaults to zero initialization."""
 
     def setup(self):
 
@@ -330,7 +288,11 @@ class DenseEquivariantFFT(Module):
 
         self.n_cells = np.product(np.asarray(self.shape))
         self.n_point = len(pt) // self.n_cells
+        if self.mask is not None:
+            self.scaled_mask = _normalise_mask(self.mask, len(pt))
 
+        # maps (n_sites) dimension of kernels to (n_point, n_point, *shape)
+        # as used in FFT-based group convolution
         self.mapping = (
             pt[: self.n_point]
             .reshape(self.n_point, self.n_cells, self.n_point)
@@ -338,16 +300,12 @@ class DenseEquivariantFFT(Module):
             .reshape(self.n_point, self.n_point, *self.shape)
         )
 
-    def make_kernel(self, kernel):
-        kernel = kernel[..., self.mapping]
-
-        return kernel
-
     @compact
     def __call__(self, x: Array) -> Array:
         """Applies the equivariant transform to the inputs along the last two
         dimensions (-2: features, -1: group elements)
         """
+        in_features = x.shape[-2]
 
         dtype = jnp.promote_types(x.dtype, self.dtype)
         x = jnp.asarray(x, dtype)
@@ -359,20 +317,19 @@ class DenseEquivariantFFT(Module):
         kernel = self.param(
             "kernel",
             self.kernel_init,
-            (
-                self.out_features,
-                self.in_features,
-                self.n_point * self.n_cells,
-            ),
+            (self.features, in_features, self.n_point * self.n_cells),
             self.dtype,
         )
 
         kernel = jnp.asarray(kernel, dtype)
 
         if self.mask is not None:
-            kernel = kernel * jnp.expand_dims(self.mask, (0, 1))
+            kernel = kernel * jnp.expand_dims(self.scaled_mask, (0, 1))
 
-        kernel = self.make_kernel(kernel)
+        # Convert the convolutional kernel of shape (features, in_features, n_symm)
+        # to the expanded kernel of shape (features, in_features, n_point(in),
+        # n_point(out), *shape) used in FFT-based group convolutions
+        kernel = kernel[..., self.mapping]
 
         x = jnp.fft.fftn(x, s=self.shape).reshape(*x.shape[:3], self.n_cells)
 
@@ -391,7 +348,7 @@ class DenseEquivariantFFT(Module):
         x = x.reshape(*x.shape[:2], -1)
 
         if self.use_bias:
-            bias = self.param("bias", self.bias_init, (self.out_features,), self.dtype)
+            bias = self.param("bias", self.bias_init, (self.features,), self.dtype)
             bias = jnp.asarray(bias, dtype)
             x += jnp.expand_dims(bias, (0, 2))
 
@@ -406,7 +363,7 @@ class DenseEquivariantIrrep(Module):
     representations of the group.
 
     Acts on a feature map of shape [batch_size, in_features, n_symm] and
-    eeturns a feature map of shape [batch_size, out_features, n_symm].
+    eeturns a feature map of shape [batch_size, features, n_symm].
     The input and the output are related by
 
     .. math ::
@@ -431,10 +388,8 @@ class DenseEquivariantIrrep(Module):
     """Irrep matrices of the symmetry group. Each element of the list is an
     array of shape [n_symm, d, d]; irreps[i][j] is the representation of the
     jth group element in irrep #i."""
-    in_features: int
-    """The number of input features; must be the second dimension of the input."""
-    out_features: int
-    """The number of output features; returned as the second dimension of the output."""
+    features: int
+    """The number of output features. Will be the second dimension of the output."""
     use_bias: bool = True
     """Whether to add a bias to the output (default: True)."""
     mask: Optional[HashableArray] = None
@@ -445,13 +400,16 @@ class DenseEquivariantIrrep(Module):
     precision: Any = None
     """numerical precision of the computation see `jax.lax.Precision`for details."""
 
-    kernel_init: NNInitFunc = unit_normal_scaling
-    """Initializer for the Dense layer matrix. Defaults to variance scaling"""
+    kernel_init: NNInitFunc = default_equivariant_initializer
+    """Initializer for the kernel. Defaults to Lecun normal."""
     bias_init: NNInitFunc = zeros
-    """Initializer for the bias. Defaults to zero initialization"""
+    """Initializer for the bias. Defaults to zero initialization."""
 
     def setup(self):
         self.n_symm = self.irreps[0].shape[0]
+        if self.mask is not None:
+            self.scaled_mask = _normalise_mask(self.mask, self.n_symm)
+
         self.forward = jnp.concatenate(
             [jnp.asarray(irrep).reshape(self.n_symm, -1) for irrep in self.irreps],
             axis=1,
@@ -530,6 +488,7 @@ class DenseEquivariantIrrep(Module):
         """Applies the equivariant transform to the inputs along the last two
         dimensions (-2: features, -1: group elements)
         """
+        in_features = x.shape[-2]
 
         dtype = jnp.promote_types(x.dtype, self.dtype)
         x = jnp.asarray(x, dtype)
@@ -539,14 +498,14 @@ class DenseEquivariantIrrep(Module):
         kernel = self.param(
             "kernel",
             self.kernel_init,
-            (self.out_features, self.in_features, self.n_symm),
+            (self.features, in_features, self.n_symm),
             self.dtype,
         )
 
         kernel = jnp.asarray(kernel, dtype)
 
         if self.mask is not None:
-            kernel = kernel * jnp.expand_dims(self.mask, (0, 1))
+            kernel = kernel * jnp.expand_dims(self.scaled_mask, (0, 1))
 
         kernel = self.forward_ft(kernel)
 
@@ -560,7 +519,7 @@ class DenseEquivariantIrrep(Module):
         x = self.inverse_ft(x)
 
         if self.use_bias:
-            bias = self.param("bias", self.bias_init, (self.out_features,), self.dtype)
+            bias = self.param("bias", self.bias_init, (self.features,), self.dtype)
             bias = jnp.asarray(bias, dtype)
 
             x += jnp.expand_dims(bias, (0, 2))
@@ -579,12 +538,9 @@ class DenseEquivariantMatrix(Module):
     """Flattened product table generated by PermutationGroup.produt_table().ravel()
     that specifies the product of the group with its involution, or the
     PermutationGroup object itself"""
-    in_features: int
-    """The number of symmetry-reduced input features. The full input size
-    is n_symm*in_features."""
-    out_features: int
+    features: int
     """The number of symmetry-reduced output features. The full output size
-    is n_symm*out_features."""
+    is n_symm*features."""
     use_bias: bool = True
     """Whether to add a bias to the output (default: True)."""
     mask: Optional[HashableArray] = None
@@ -594,33 +550,15 @@ class DenseEquivariantMatrix(Module):
     precision: Any = None
     """numerical precision of the computation see `jax.lax.Precision`for details."""
 
-    kernel_init: NNInitFunc = unit_normal_scaling
-    """Initializer for the Dense layer matrix. Defaults to variance scaling"""
+    kernel_init: NNInitFunc = default_equivariant_initializer
+    """Initializer for the kernel. Defaults to Lecun normal."""
     bias_init: NNInitFunc = zeros
-    """Initializer for the bias. Defaults to zero initialization"""
+    """Initializer for the bias. Defaults to zero initialization."""
 
     def setup(self):
-
         self.n_symm = np.asarray(self.product_table).shape[0]
-
-        self.stdev = 1.0 / np.sqrt(self.in_features * self.n_symm)
-
-    def full_kernel(self, kernel):
-        """
-        Converts the symmetry-reduced kernel of shape (n_sites, features) to
-        the full Dense kernel of shape (n_sites, features * n_symm).
-        """
-
-        result = jnp.take(kernel, jnp.asarray(self.product_table).ravel(), 2)
-
-        result = result.reshape(
-            self.out_features, self.in_features, self.n_symm, self.n_symm
-        )
-        result = result.transpose(1, 2, 0, 3).reshape(
-            self.n_symm * self.in_features, -1
-        )
-
-        return result
+        if self.mask is not None:
+            self.scaled_mask = _normalise_mask(self.mask, self.n_symm)
 
     @compact
     def __call__(self, x: Array) -> Array:
@@ -630,38 +568,42 @@ class DenseEquivariantMatrix(Module):
         Returns:
           The transformed input.
         """
+        in_features = x.shape[-2]
+
         dtype = jnp.promote_types(x.dtype, self.dtype)
         x = jnp.asarray(x, dtype)
 
-        x = x.reshape(-1, x.shape[1] * x.shape[2])
-
         kernel = self.param(
             "kernel",
             self.kernel_init,
-            (self.out_features, self.in_features, self.n_symm),
+            (self.features, in_features, self.n_symm),
             self.dtype,
         )
 
         kernel = jnp.asarray(kernel, dtype)
 
         if self.mask is not None:
-            kernel = kernel * jnp.expand_dims(self.mask, (0, 1))
+            kernel = kernel * jnp.expand_dims(self.scaled_mask, (0, 1))
 
-        kernel = self.full_kernel(kernel)
+        # Converts the convolutional kernel of shape (features, in_features, n_symm)
+        # to a full dense kernel of shape (features, in_features, n_symm, n_symm)
+        # result[out, in, g, h] == kernel[out, in, g^{-1}h]
+        # input dimensions are [in, g], output dimensions are [out, h]
+        kernel = jnp.take(kernel, jnp.asarray(self.product_table), 2)
         kernel = jnp.asarray(kernel, dtype)
 
         x = lax.dot_general(
             x,
             kernel,
-            (((x.ndim - 1,), (0,)), ((), ())),
+            (((x.ndim - 2, x.ndim - 1), (1, 2)), ((), ())),
             precision=self.precision,
         )
 
-        x = x.reshape(-1, self.out_features, self.n_symm)
+        x = x.reshape(-1, self.features, self.n_symm)
 
         if self.use_bias:
-            bias = self.param("bias", self.bias_init, (self.out_features,), self.dtype)
-            x += jnp.expand_dims(bias, (0, 2))
+            bias = self.param("bias", self.bias_init, (self.features,), self.dtype)
+            x += jnp.expand_dims(bias, 1)
 
         return x
 
@@ -672,13 +614,18 @@ def DenseSymm(symmetries, point_group=None, mode="auto", shape=None, **kwargs):
     equivariant with respect to the symmetry operations in the group and can
     be averaged to produce an invariant model.
 
+    This layer maps an input of shape `(..., in_features, n_sites)` to an
+    output of shape `(..., features, num_symm)`.
+
     Note: The output shape has changed to seperate the feature and symmetry
     dimensions. The previous shape was [num_samples, num_symm*features] and
-    the new shape is [num_samples, num_symm, features]
+    the new shape is [num_samples, features, num_symm]
 
     Args:
         symmetries: A specification of the symmetry group. Can be given by a
-            nk.graph.Graph, a nk.utils.PermuationGroup, or an array [n_symm, n_sites]
+            :ref:`netket.graph.Graph`, a :ref:`netket.utils.group.PermutationGroup`, or an array
+            of shape :code:`(n_symm, n_sites)`. A :ref:`netket.utils.HashableArray` may also
+            be passed.
             specifying the permutations corresponding to symmetry transformations
             of the lattice.
         point_group: The point group, from which the space group is built.
@@ -687,8 +634,8 @@ def DenseSymm(symmetries, point_group=None, mode="auto", shape=None, **kwargs):
             transform, matrix multiplication, or to choose a sensible default
             based on the symmetry group.
         shape: A tuple specifying the dimensions of the translation group.
-        features: The number of symmetry-reduced features. The full output size
-            is [n_symm,features].
+        features: The number of output features. The full output shape
+            is [n_batch,features,n_symm].
         use_bias: A bool specifying whether to add a bias to the output (default: True).
         mask: An optional array of shape [n_sites] consisting of ones and zeros
             that can be used to give the kernel a particular shape.
@@ -698,7 +645,6 @@ def DenseSymm(symmetries, point_group=None, mode="auto", shape=None, **kwargs):
         kernel_init: Optional kernel initialization function. Defaults to variance scaling.
         bias_init: Optional bias initialization function. Defaults to zero initialization.
     """
-
     if isinstance(symmetries, Lattice) and (
         point_group is not None or symmetries._point_group is not None
     ):
@@ -713,12 +659,10 @@ def DenseSymm(symmetries, point_group=None, mode="auto", shape=None, **kwargs):
                 "in order to construct the space group"
             )
         sym = HashableArray(np.asarray(symmetries.automorphisms()))
-    elif isinstance(symmetries, PermutationGroup) or hasattr(symmetries, "__len__"):
-        sym = HashableArray(np.asarray(symmetries))
+    elif isinstance(symmetries, HashableArray):
+        sym = symmetries
     else:
-        raise ValueError(
-            "Symmetries must be specified as a Graph, PermutationGroup or Array"
-        )
+        sym = HashableArray(np.asarray(symmetries))
 
     if mode == "fft":
         if shape is None:
@@ -737,11 +681,19 @@ def DenseSymm(symmetries, point_group=None, mode="auto", shape=None, **kwargs):
         )
 
 
-def DenseEquivariant(symmetries, mode="auto", shape=None, point_group=None, **kwargs):
+def DenseEquivariant(
+    symmetries,
+    features: int = None,
+    mode="auto",
+    shape=None,
+    point_group=None,
+    in_features=None,
+    **kwargs,
+):
     r"""A group convolution operation that is equivariant over a symmetry group.
 
     Acts on a feature map of symmetry poses of shape [num_samples, in_features, num_symm]
-    and returns a feature  map of poses of shape [num_samples, out_features, num_symm]
+    and returns a feature  map of poses of shape [num_samples, features, num_symm]
 
     G-convolutions are described in ` Cohen et. {\it al} <http://proceedings.mlr.press/v48/cohenc16.pdf>`_
     and applied to quantum many-body problems in ` Roth et. {\it al} <https://arxiv.org/pdf/2104.05085.pdf>`_
@@ -755,6 +707,9 @@ def DenseEquivariant(symmetries, mode="auto", shape=None, point_group=None, **kw
     Group elements that differ by the same symmetry operation (i.e. :math:`g = xh`
     and :math:`g' = xh'`) are connected by the same filter.
 
+    This layer maps an input of shape `(..., in_features, n_sites)` to an
+    output of shape `(..., features, num_symm)`.
+
     Args:
         symmetries: A specification of the symmetry group. Can be given by a
             nk.graph.Graph, an nk.utils.PermuationGroup, a list of irreducible
@@ -765,10 +720,8 @@ def DenseEquivariant(symmetries, mode="auto", shape=None, point_group=None, **kw
             fourier transform over the translation group, a fourier transform using
             the irreducible representations or by constructing the full kernel matrix.
         shape: A tuple specifying the dimensions of the translation group.
-        in_features: The number of symmetry-reduced features. The full input size
-            is n_symm*in_features.
-        out_features: The number of symmetry-reduced features. The full output size
-            is n_symm*out_features.
+        features: The number of output features. The full output shape
+            is [n_batch,features,n_symm].
         use_bias: A bool specifying whether to add a bias to the output (default: True).
         mask: An optional array of shape [n_sites] consisting of ones and zeros
             that can be used to give the kernel a particular shape.
@@ -778,6 +731,29 @@ def DenseEquivariant(symmetries, mode="auto", shape=None, point_group=None, **kw
         kernel_init: Optional kernel initialization function. Defaults to variance scaling.
         bias_init: Optional bias initialization function. Defaults to zero initialization.
     """
+    # deprecate in_features
+    if in_features is not None:
+        warn_deprecation(
+            (
+                "`in_features` is now automatically detected from the input and deprecated."
+                "Please remove it when calling `DenseEquivariant`."
+            )
+        )
+    if "out_features" in kwargs:
+        warn_deprecation(
+            "`out_features` has been renamed to `features` and the old name is "
+            "now deprecated. Please update your code."
+        )
+        if features is not None:
+            raise ValueError(
+                "You must only specify `features`. `out_features` is deprecated."
+            )
+        features = kwargs.pop("out_features")
+
+    if features is None:
+        raise ValueError("`features` not specified (the number of output features).")
+
+    kwargs["features"] = features
 
     if isinstance(symmetries, Lattice) and (
         point_group is not None or symmetries._point_group is not None
