@@ -24,6 +24,11 @@ from ._hamiltonian import Ising, Heisenberg, BoseHubbard
 from ._abstract_super_operator import AbstractSuperOperator
 from ._local_liouvillian import LocalLiouvillian
 
+from ._continuous_operator import ContinuousOperator
+from ._kinetic import KineticEnergy
+from ._potential import PotentialEnergy
+from ._sumoperators import SumOperator
+
 from . import spin, boson
 
 # TODO: Deprecated. Remove in v3.1
@@ -37,3 +42,7 @@ from ._local_cost_functions import (
     local_value_cost,
     local_value_op_op_cost,
 )
+
+from netket.utils import _auto_export
+
+_auto_export(__name__)
