@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import List, Sequence, Union
 from numba import jit
 
 import numpy as np
@@ -145,7 +146,10 @@ class Ising(SpecialHamiltonian):
 
         self._h = dtype(h)
         self._J = dtype(J)
-        self._edges = np.asarray(list(graph.edges()), dtype=np.intp)
+        self._edges = np.asarray(
+            [[u, v] for u, v in graph.edges()],
+            dtype=np.intp,
+        )
 
         self._dtype = dtype
 
@@ -270,23 +274,7 @@ class Ising(SpecialHamiltonian):
         for i in range(x.shape[0]):
             mels[diag_ind] = 0.0
             for k in range(edges.shape[0]):
-                mels[diag_ind] += (
-                    J
-                    * x[
-                        i,
-                        edges[
-                            k,
-                            0,
-                        ],
-                    ]
-                    * x[
-                        i,
-                        edges[
-                            k,
-                            1,
-                        ],
-                    ]
-                )
+                mels[diag_ind] += J * x[i, edges[k, 0]] * x[i, edges[k, 1]]
 
             odiag_ind = 1 + diag_ind
 
@@ -363,8 +351,10 @@ class Heisenberg(GraphOperator):
         self,
         hilbert: AbstractHilbert,
         graph: AbstractGraph,
-        J: float = 1,
+        J: Union[float, Sequence[float]] = 1.0,
         sign_rule=None,
+        *,
+        acting_on_subspace: Union[List[int], int] = None,
     ):
         """
         Constructs an Heisenberg operator given a hilbert space and a graph providing the
@@ -374,11 +364,20 @@ class Heisenberg(GraphOperator):
             hilbert: Hilbert space the operator acts on.
             graph: The graph upon which this hamiltonian is defined.
             J: The strength of the coupling. Default is 1.
-            sign_rule: If enabled, Marshal's sign rule will be used. On a bipartite
-                       lattice, this corresponds to a basis change flipping the Sz direction
-                       at every odd site of the lattice. For non-bipartite lattices, the
-                       sign rule cannot be applied. Defaults to True if the lattice is
-                       bipartite, False otherwise.
+               Can pass a sequence of coupling strengths with coloured graphs:
+               edges of colour n will have coupling strength J[n]
+            sign_rule: If True, Marshal's sign rule will be used. On a bipartite
+                lattice, this corresponds to a basis change flipping the Sz direction
+                at every odd site of the lattice. For non-bipartite lattices, the
+                sign rule cannot be applied. Defaults to True if the lattice is
+                bipartite, False otherwise.
+                If a sequence of coupling strengths is passed, defaults to False
+                and a matching sequence of sign_rule must be specified to override it
+            acting_on_subspace: Specifies the mapping between nodes of the graph and
+                Hilbert space sites, so that graph node :code:`i ∈ [0, ..., graph.n_nodes - 1]`,
+                corresponds to :code:`acting_on_subspace[i] ∈ [0, ..., hilbert.n_sites]`.
+                Must be a list of length `graph.n_nodes`. Passing a single integer :code:`start`
+                is equivalent to :code:`[start, ..., start + graph.n_nodes - 1]`.
 
         Examples:
          Constructs a ``Heisenberg`` operator for a 1D system.
@@ -388,10 +387,27 @@ class Heisenberg(GraphOperator):
             >>> hi = nk.hilbert.Spin(s=0.5, total_sz=0, N=g.n_nodes)
             >>> op = nk.operator.Heisenberg(hilbert=hi, graph=g)
             >>> print(op)
-            Heisenberg(J=1, sign_rule=True; dim=20)
+            Heisenberg(J=1.0, sign_rule=True; dim=20)
         """
-        if sign_rule is None:
-            sign_rule = graph.is_bipartite()
+        if isinstance(J, Sequence):
+            # check that the number of Js matches the number of colours
+            assert len(J) == max(graph.edge_colors) + 1
+
+            if sign_rule is None:
+                sign_rule = [False] * len(J)
+            else:
+                assert len(sign_rule) == len(J)
+                for i in range(len(J)):
+                    subgraph = Graph(edges=graph.edges(filter_color=i))
+                    if sign_rule[i] and not subgraph.is_bipartite():
+                        raise ValueError(
+                            "sign_rule=True specified for a non-bipartite lattice"
+                        )
+        else:
+            if sign_rule is None:
+                sign_rule = graph.is_bipartite()
+            elif sign_rule and not graph.is_bipartite():
+                raise ValueError("sign_rule=True specified for a non-bipartite lattice")
 
         self._J = J
         self._sign_rule = sign_rule
@@ -412,17 +428,23 @@ class Heisenberg(GraphOperator):
                 [0, 0, 0, 0],
             ]
         )
-        if sign_rule:
-            if not graph.is_bipartite():
-                raise ValueError("sign_rule=True specified for a non-bipartite lattice")
-            heis_term = sz_sz - exchange
+
+        if isinstance(J, Sequence):
+            bond_ops = [
+                J[i] * (sz_sz - exchange if sign_rule[i] else sz_sz + exchange)
+                for i in range(len(J))
+            ]
+            bond_ops_colors = list(range(len(J)))
         else:
-            heis_term = sz_sz + exchange
+            bond_ops = [J * (sz_sz - exchange if sign_rule else sz_sz + exchange)]
+            bond_ops_colors = []
 
         super().__init__(
             hilbert,
             graph,
-            bond_ops=[J * heis_term],
+            bond_ops=bond_ops,
+            bond_ops_colors=bond_ops_colors,
+            acting_on_subspace=acting_on_subspace,
         )
 
     @property
@@ -772,6 +794,9 @@ class BoseHubbard(SpecialHamiltonian):
         if self._max_mels.size < total_size:
             self._max_mels = np.empty(total_size, dtype=self._max_mels.dtype)
             self._max_xprime = np.empty((total_size, x.shape[1]), dtype=x.dtype)
+        else:
+            if x.dtype != self._max_xprime.dtype:
+                self._max_xprime = self._max_xprime.astype(x.dtype)
 
         return self._flattened_kernel(
             np.asarray(x),
