@@ -12,15 +12,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Any, Optional, Callable
+from functools import partial
+from typing import Any, Callable, Optional, Tuple, Union
 
 import jax
+from flax import linen as nn
 from jax import numpy as jnp
 from jax.experimental import loops
 
-from netket.hilbert import AbstractParticle
+from netket.hilbert import ContinuousHilbert
 
-from netket.utils import mpi
+from netket.utils import mpi, wrap_afun
 from netket.utils.types import PyTree, PRNGKeyT
 
 from netket.utils.deprecation import deprecated, warn_deprecation
@@ -32,39 +34,40 @@ from .base import Sampler, SamplerState
 @struct.dataclass
 class MetropolisRule:
     """
-    Base class for Transition rules of Metropolis, such as Local, Exchange, Hamiltonian
+    Base class for transition rules of Metropolis, such as Local, Exchange, Hamiltonian
     and several others.
     """
 
     def init_state(
         rule,
-        sampler: Sampler,
-        machine: Callable,
+        sampler: "MetropolisSampler",
+        machine: nn.Module,
         params: PyTree,
         key: PRNGKeyT,
     ) -> Optional[Any]:
         """
-        Initialises the optional internal state of the Metropolis Sampler Transition
-        Rule.
+        Initialises the optional internal state of the Metropolis sampler transition
+        rule.
 
         The provided key is unique and does not need to be splitted.
-        It should return an immutable datastructure.
+
+        It should return an immutable data structure.
 
         Arguments:
-            sampler: The Metropolis sampler
-            machine: The forward evaluation function of the model, accepting PyTrees of parameters and inputs.
-            params: The dict of variables needed to evaluate the model.
-            key: A Jax PRNGKey rng state.
+            sampler: The Metropolis sampler.
+            machine: A Flax module with the forward pass of the log-pdf.
+            params: The PyTree of parameters of the model.
+            key: A Jax PRNGKey.
 
         Returns:
-            An Optional State.
+            An optional state.
         """
         return None
 
     def reset(
         rule,
-        sampler: Sampler,
-        machine: Callable,
+        sampler: "MetropolisSampler",
+        machine: nn.Module,
         params: PyTree,
         sampler_state: SamplerState,
     ) -> Optional[Any]:
@@ -72,33 +75,32 @@ class MetropolisRule:
         Resets the internal state of the Metropolis Sampler Transition Rule.
 
         Arguments:
-            sampler: The Metropolis sampler
-            machine: The forward evaluation function of the model, accepting PyTrees of parameters and inputs.
-            params: The dict of variables needed to evaluate the model.
+            sampler: The Metropolis sampler.
+            machine: A Flax module with the forward pass of the log-pdf.
+            params: The PyTree of parameters of the model.
             sampler_state: The current state of the sampler. Should not modify it.
 
         Returns:
-           A new, resetted, state of the rule. This returns the same type of :py:meth:`sampler_state.rule_state` and might be None.
-
+           A new, resetted, state of the rule. This returns the same type of :py:meth:`sampler_state.rule_state` and might be `None`.
         """
         return sampler_state.rule_state
 
     def transition(
         rule,
-        sampler: Sampler,
-        machine: Callable,
+        sampler: "MetropolisSampler",
+        machine: nn.Module,
         parameters: PyTree,
         state: SamplerState,
         key: PRNGKeyT,
         σ: jnp.ndarray,
-    ) -> jnp.ndarray:
+    ) -> Tuple[jnp.ndarray, Optional[jnp.ndarray]]:
 
-        raise NotImplementedError("error")
+        raise NotImplementedError
 
     def random_state(
         rule,
-        sampler: Sampler,
-        machine: Callable,
+        sampler: "MetropolisSampler",
+        machine: nn.Module,
         parameters: PyTree,
         state: SamplerState,
         key: PRNGKeyT,
@@ -109,11 +111,11 @@ class MetropolisRule:
         By default this calls :func:`netket.hilbert.random.random_state`.
 
         Arguments:
-            sampler: the sampler
-            machine: the function to evaluate the model
-            parameters: the parameters of the model
-            state: the current sampler state
-            key: the PRNGKey to use to generate the random state
+            sampler: The Metropolis sampler.
+            machine: A Flax module with the forward pass of the log-pdf.
+            parameters: The PyTree of parameters of the model.
+            state: The current state of the sampler. Should not modify it.
+            key: The PRNGKey to use to generate the random state.
         """
         return sampler.hilbert.random_state(
             key, size=sampler.n_batches, dtype=sampler.dtype
@@ -123,20 +125,28 @@ class MetropolisRule:
 @struct.dataclass
 class MetropolisSamplerState(SamplerState):
     """
-    State for a metropolis sampler.
-    Contains the current configuration, the rng state and the (optional)
+    State for a Metropolis sampler.
+
+    Contains the current configuration, the RNG state and the (optional)
     state of the transition rule.
     """
 
     σ: jnp.ndarray
-    """Current batch of configurations in the markov chain."""
+    """Current batch of configurations in the Markov chain."""
     rng: jnp.ndarray
     """State of the random number generator (key, in jax terms)."""
     rule_state: Optional[Any]
-    """Optional state of a transition rule."""
-    n_steps_proc: int = 0
+    """Optional state of the transition rule."""
+
+    # those are initialised to 0. We want to initialise them to zero arrays because they can
+    # be passed to jax jitted functions that require type invariance to avoid recompilation
+    n_steps_proc: int = struct.field(
+        default_factory=lambda: jnp.zeros((), dtype=jnp.int64)
+    )
     """Number of moves performed along the chains in this process since the last reset."""
-    n_accepted_proc: int = 0
+    n_accepted_proc: int = struct.field(
+        default_factory=lambda: jnp.zeros((), dtype=jnp.int64)
+    )
     """Number of accepted transitions among the chains in this process since the last reset."""
 
     @property
@@ -196,16 +206,16 @@ class MetropolisSamplerState(SamplerState):
 @struct.dataclass
 class MetropolisSampler(Sampler):
     r"""
-    Metropolis-Hastings sampler for an Hilbert space according to a specific transition rule.
+    Metropolis-Hastings sampler for a Hilbert space according to a specific transition rule.
 
     The transition rule is used to generate a proposed state :math:`s^\prime`, starting from the
     current state :math:`s`. The move is accepted with probability
 
     .. math::
 
-        A(s \rightarrow s^\prime) = \mathrm{min} \left( 1,\frac{P(s^\prime)}{P(s)} F(e^{L(s,s^\prime)}) \right) ,
+        A(s \rightarrow s^\prime) = \mathrm{min} \left( 1,\frac{P(s^\prime)}{P(s)} e^{L(s,s^\prime)} \right) ,
 
-    where the probability being sampled from is :math:`P(s)=|M(s)|^p. Here ::math::`M(s)` is a
+    where the probability being sampled from is :math:`P(s)=|M(s)|^p`. Here :math:`M(s)` is a
     user-provided function (the machine), :math:`p` is also user-provided with default value :math:`p=2`,
     and :math:`L(s,s^\prime)` is a suitable correcting factor computed by the transition kernel.
 
@@ -213,28 +223,31 @@ class MetropolisSampler(Sampler):
     """
 
     rule: MetropolisRule = None
-    """The metropolis transition rule."""
+    """The Metropolis transition rule."""
     n_sweeps: int = struct.field(pytree_node=False, default=None)
-    """Number of sweeps for each step along the chain. Defaults to number of sites in hilbert space."""
+    """Number of sweeps for each step along the chain. Defaults to the number of sites in the Hilbert space."""
     reset_chains: bool = struct.field(pytree_node=False, default=False)
-    """If True resets the chain state when reset is called (every new sampling)."""
+    """If True, resets the chain state when `reset` is called on every new sampling."""
 
     def __pre_init__(self, hilbert, rule, **kwargs):
-        r"""
+        """
         Constructs a Metropolis Sampler.
 
         Args:
-            hilbert: The hilbert space to sample
+            hilbert: The Hilbert space to sample.
             rule: A `MetropolisRule` to generate random transitions from a given state as
                     well as uniform random states.
-            n_sweeps: The number of exchanges that compose a single sweep.
-                    If None, sweep_size is equal to the number of degrees of freedom being sampled
-                    (the size of the input vector s to the machine).
-            reset_chains: If False the state configuration is not resetted when reset() is called.
-            n_chains: The number of Markov Chain to be run in parallel on a single process.
+            n_chains: The total number of independent Markov chains across all MPI ranks. Either specify this or `n_chains_per_rank`.
+            n_chains_per_rank: Number of independent chains on every MPI rank (default = 16).
+            n_sweeps: Number of sweeps for each step along the chain. Defaults to the number of sites in the Hilbert space.
+                    This is equivalent to subsampling the Markov chain.
+            reset_chains: If True, resets the chain state when `reset` is called on every new sampling (default = False).
             machine_pow: The power to which the machine should be exponentiated to generate the pdf (default = 2).
-            dtype: The dtype of the statees sampled (default = np.float32).
+            dtype: The dtype of the states sampled (default = np.float64).
         """
+        if "n_chains" not in kwargs and "n_chains_per_rank" not in kwargs:
+            kwargs["n_chains_per_rank"] = 16
+
         # process arguments in the base
         args, kwargs = super().__pre_init__(hilbert=hilbert, **kwargs)
 
@@ -258,10 +271,38 @@ class MetropolisSampler(Sampler):
         if not isinstance(self.reset_chains, bool):
             raise TypeError("reset_chains must be a boolean.")
 
-        #  Default value of n_sweeps
+        # Default value of n_sweeps
         if self.n_sweeps is None:
             object.__setattr__(self, "n_sweeps", self.hilbert.size)
 
+    def sample_next(
+        sampler,
+        machine: Union[Callable, nn.Module],
+        parameters: PyTree,
+        state: Optional[SamplerState] = None,
+    ) -> Tuple[SamplerState, jnp.ndarray]:
+        """
+        Samples the next state in the Markov chain.
+
+        Args:
+            machine: A Flax module or callable with the forward pass of the log-pdf.
+                If it is a callable, it should have the signature :code:`f(parameters, σ) -> jnp.ndarray`.
+            parameters: The PyTree of parameters of the model.
+            state: The current state of the sampler. If not specified, then initialize and reset it.
+
+        Returns:
+            state: The new state of the sampler.
+            σ: The next batch of samples.
+
+        Note:
+            The return order is inverted wrt `sample` because when called inside of
+            a scan function the first returned argument should be the state.
+        """
+        if state is None:
+            state = sampler.reset(machine, parameters)
+
+        return sampler._sample_next(wrap_afun(machine), parameters, state)
+
     def _init_state(sampler, machine, params, key):
         key_state, key_rule = jax.random.split(key, 2)
         rule_state = sampler.rule.init_state(sampler, machine, params, key_rule)
@@ -295,6 +336,12 @@ class MetropolisSampler(Sampler):
         )
 
     def _sample_next(sampler, machine, parameters, state):
+        """
+        Implementation of `sample_next` for subclasses of `MetropolisSampler`.
+
+        If you subclass `MetropolisSampler`, you should override this and not `sample_next`
+        itself, because `sample_next` contains some common logic.
+        """
         new_rng, rng = jax.random.split(state.rng)
 
         with loops.Scope() as s:
@@ -342,15 +389,18 @@ class MetropolisSampler(Sampler):
 
         return new_state, new_state.σ
 
+    def _sample_chain(sampler, machine, parameters, state, chain_length):
+        return _sample_chain(sampler, machine, parameters, state, chain_length)
+
     def __repr__(sampler):
         return (
             f"{type(sampler).__name__}("
             + "\n  hilbert = {},".format(sampler.hilbert)
             + "\n  rule = {},".format(sampler.rule)
             + "\n  n_chains = {},".format(sampler.n_chains)
-            + "\n  machine_power = {},".format(sampler.machine_pow)
-            + "\n  reset_chains = {},".format(sampler.reset_chains)
             + "\n  n_sweeps = {},".format(sampler.n_sweeps)
+            + "\n  reset_chains = {},".format(sampler.reset_chains)
+            + "\n  machine_power = {},".format(sampler.machine_pow)
             + "\n  dtype = {}".format(sampler.dtype)
             + ")"
         )
@@ -360,12 +410,75 @@ class MetropolisSampler(Sampler):
             f"{type(sampler).__name__}("
             + "rule = {}, ".format(sampler.rule)
             + "n_chains = {}, ".format(sampler.n_chains)
-            + "machine_power = {}, ".format(sampler.machine_pow)
             + "n_sweeps = {}, ".format(sampler.n_sweeps)
+            + "reset_chains = {}, ".format(sampler.reset_chains)
+            + "machine_power = {}, ".format(sampler.machine_pow)
             + "dtype = {})".format(sampler.dtype)
         )
 
 
+@deprecated(
+    "The module function `sample_next` is deprecated in favor of the class method `sample_next`."
+)
+def sample_next(
+    sampler: MetropolisSampler,
+    machine: Union[Callable, nn.Module],
+    parameters: PyTree,
+    state: Optional[SamplerState] = None,
+) -> Tuple[SamplerState, jnp.ndarray]:
+    """
+    Samples the next state in the Markov chain.
+
+    Args:
+        sampler: The Metropolis sampler.
+        machine: A Flax module or callable with the forward pass of the log-pdf.
+            If it is a callable, it should have the signature :code:`f(parameters, σ) -> jnp.ndarray`.
+        parameters: The PyTree of parameters of the model.
+        state: The current state of the sampler. If not specified, then initialize and reset it.
+
+    Returns:
+        state: The new state of the sampler.
+        σ: The next batch of samples.
+    """
+    return sampler.sample_next(machine, parameters, state)
+
+
+@partial(jax.jit, static_argnums=(1, 4))
+def _sample_chain(
+    sampler: MetropolisSampler,
+    machine: nn.Module,
+    parameters: PyTree,
+    state: SamplerState,
+    chain_length: int,
+) -> Tuple[jnp.ndarray, SamplerState]:
+    """
+    Samples `chain_length` batches of samples along the chains.
+
+    Internal method used for jitting calls.
+
+    Arguments:
+        sampler: The Monte Carlo sampler.
+        machine: A Flax module with the forward pass of the log-pdf.
+        parameters: The PyTree of parameters of the model.
+        state: The current state of the sampler.
+        chain_length: The length of the chains.
+
+    Returns:
+        σ: The next batch of samples.
+        state: The new state of the sampler
+    """
+    _sample_next = lambda state, _: sampler.sample_next(machine, parameters, state)
+
+    state, samples = jax.lax.scan(
+        _sample_next,
+        state,
+        xs=None,
+        length=chain_length,
+    )
+
+    return samples, state
+
+
 def MetropolisLocal(hilbert, *args, **kwargs) -> MetropolisSampler:
     r"""
     Sampler acting on one local degree of freedom.
@@ -379,7 +492,7 @@ def MetropolisLocal(hilbert, *args, **kwargs) -> MetropolisSampler:
 
     1. One of the site indices :math:`i = 1\dots N` is chosen with uniform probability.
 
-    2. Among all the possible (:math:`m`) values that :math:`s_i` can take,
+    2. Among all the possible (:math:`m - 1`) values that :math:`s^\prime_i` can take,
     one of them is chosen with uniform probability.
 
     For example, in the case of spin :math:`1/2` particles, :math:`m=2`
@@ -389,17 +502,17 @@ def MetropolisLocal(hilbert, *args, **kwargs) -> MetropolisSampler:
     In the case of bosons, with occupation numbers
     :math:`s_i = 0, 1, \dots n_{\mathrm{max}}`, :class:`MetropolisLocal`
     would pick a random local occupation number uniformly between :math:`0`
-    and :math:`n_{\mathrm{max}}`.
+    and :math:`n_{\mathrm{max}}` except the current :math:`s_i`.
 
     Args:
-        hilbert: The hilbert space to sample
-        n_chains: The number of Markov Chain to be run in parallel on a single process.
-        n_sweeps: The number of exchanges that compose a single sweep.
-                If None, sweep_size is equal to the number of degrees of freedom being sampled
-                (the size of the input vector s to the machine).
-        n_chains: The number of batches of the states to sample (default = 8)
+        hilbert: The Hilbert space to sample.
+        n_chains: The total number of independent Markov chains across all MPI ranks. Either specify this or `n_chains_per_rank`.
+        n_chains_per_rank: Number of independent chains on every MPI rank (default = 16).
+        n_sweeps: Number of sweeps for each step along the chain. Defaults to the number of sites in the Hilbert space.
+                This is equivalent to subsampling the Markov chain.
+        reset_chains: If True, resets the chain state when `reset` is called on every new sampling (default = False).
         machine_pow: The power to which the machine should be exponentiated to generate the pdf (default = 2).
-        dtype: The dtype of the statees sampled (default = np.float32).
+        dtype: The dtype of the states sampled (default = np.float64).
     """
     from .rules import LocalRule
 
@@ -425,7 +538,6 @@ def MetropolisExchange(
 
     2. The sites are exchanged, i.e. :math:`s^\prime_i = s_j` and :math:`s^\prime_j = s_i`.
 
-
     Notice that this sampling method generates random permutations of the quantum
     numbers, thus global quantities such as the sum of the local quantum numbers
     are conserved during the sampling.
@@ -434,16 +546,15 @@ def MetropolisExchange(
     otherwise the sampling would be strongly not ergodic.
 
     Args:
-        hilbert: The hilbert space to sample
+        hilbert: The Hilbert space to sample.
         d_max: The maximum graph distance allowed for exchanges.
-        n_chains: The number of Markov Chain to be run in parallel on a single process.
-        n_sweeps: The number of exchanges that compose a single sweep.
-                If None, sweep_size is equal to the number of degrees of freedom being sampled
-                (the size of the input vector s to the machine).
-        n_batches: The number of batches of the states to sample (default = 8)
+        n_chains: The total number of independent Markov chains across all MPI ranks. Either specify this or `n_chains_per_rank`.
+        n_chains_per_rank: Number of independent chains on every MPI rank (default = 16).
+        n_sweeps: Number of sweeps for each step along the chain. Defaults to the number of sites in the Hilbert space.
+                This is equivalent to subsampling the Markov chain.
+        reset_chains: If True, resets the chain state when `reset` is called on every new sampling (default = False).
         machine_pow: The power to which the machine should be exponentiated to generate the pdf (default = 2).
-        dtype: The dtype of the statees sampled (default = np.float32).
-
+        dtype: The dtype of the states sampled (default = np.float64).
 
     Examples:
           Sampling from a RBM machine in a 1D lattice of spin 1/2, using
@@ -457,7 +568,7 @@ def MetropolisExchange(
           >>> # Construct a MetropolisExchange Sampler
           >>> sa = nk.sampler.MetropolisExchange(hi, graph=g)
           >>> print(sa)
-          MetropolisSampler(rule = ExchangeRule(# of clusters: 200), n_chains = 16, machine_power = 2, n_sweeps = 100, dtype = <class 'numpy.float64'>)
+          MetropolisSampler(rule = ExchangeRule(# of clusters: 200), n_chains = 16, n_sweeps = 100, reset_chains = False, machine_power = 2, dtype = <class 'numpy.float64'>)
     """
     from .rules import ExchangeRule
 
@@ -485,15 +596,15 @@ def MetropolisHamiltonian(hilbert, hamiltonian, *args, **kwargs) -> MetropolisSa
     you should use :class:`netket.sampler.MetropolisHamiltonianNumpy`
 
     Args:
-       machine: A machine :math:`\Psi(s)` used for the sampling.
-                The probability distribution being sampled
-                from is :math:`F(\Psi(s))`, where the function
-                :math:`F(X)`, is arbitrary, by default :math:`F(X)=|X|^2`.
-       hamiltonian: The operator used to perform off-diagonal transition.
-       n_chains: The number of Markov Chain to be run in parallel on a single process.
-       sweep_size: The number of exchanges that compose a single sweep.
-                   If None, sweep_size is equal to the number of degrees of freedom (n_visible).
-
+        hilbert: The Hilbert space to sample.
+        hamiltonian: The operator used to perform off-diagonal transition.
+        n_chains: The total number of independent Markov chains across all MPI ranks. Either specify this or `n_chains_per_rank`.
+        n_chains_per_rank: Number of independent chains on every MPI rank (default = 16).
+        n_sweeps: Number of sweeps for each step along the chain. Defaults to the number of sites in the Hilbert space.
+                This is equivalent to subsampling the Markov chain.
+        reset_chains: If True, resets the chain state when `reset` is called on every new sampling (default = False).
+        machine_pow: The power to which the machine should be exponentiated to generate the pdf (default = 2).
+        dtype: The dtype of the states sampled (default = np.float64).
 
     Examples:
        Sampling from a RBM machine in a 1D lattice of spin 1/2
@@ -506,10 +617,10 @@ def MetropolisHamiltonian(hilbert, hamiltonian, *args, **kwargs) -> MetropolisSa
        >>> # Transverse-field Ising Hamiltonian
        >>> ha = nk.operator.Ising(hilbert=hi, h=1.0, graph=g)
        >>>
-       >>> # Construct a MetropolisExchange Sampler
+       >>> # Construct a MetropolisHamiltonian Sampler
        >>> sa = nk.sampler.MetropolisHamiltonian(hi, hamiltonian=ha)
        >>> print(sa)
-       MetropolisSampler(rule = HamiltonianRule(Ising(J=1.0, h=1.0; dim=100)), n_chains = 16, machine_power = 2, n_sweeps = 100, dtype = <class 'numpy.float64'>)
+       MetropolisSampler(rule = HamiltonianRule(Ising(J=1.0, h=1.0; dim=100)), n_chains = 16, n_sweeps = 100, reset_chains = False, machine_power = 2, dtype = <class 'numpy.float64'>)
     """
     from .rules import HamiltonianRule
 
@@ -518,17 +629,22 @@ def MetropolisHamiltonian(hilbert, hamiltonian, *args, **kwargs) -> MetropolisSa
 
 
 def MetropolisGaussian(hilbert, sigma=1.0, *args, **kwargs) -> MetropolisSampler:
-    r"""This sampler acts on all particle positions simultaneously
+    """This sampler acts on all particle positions simultaneously
     and proposes a new state according to a Gaussian distribution
-    with width sigma.
+    with width `sigma`.
+
     Args:
-       hilber: The continuous Hilbert space
-       sigma: The width of the Gaussian proposal distribution (default: 1.0)
-       n_chains: The number of Markov Chain to be run in parallel on a single process.
-       sweep_size: The number of exchanges that compose a single sweep.
-                   If None, sweep_size is equal to the number of degrees of freedom (n_visible).
+        hilbert: The continuous Hilbert space to sample.
+        sigma: The width of the Gaussian proposal distribution (default = 1.0).
+        n_chains: The total number of independent Markov chains across all MPI ranks. Either specify this or `n_chains_per_rank`.
+        n_chains_per_rank: Number of independent chains on every MPI rank (default = 16).
+        n_sweeps: Number of sweeps for each step along the chain. Defaults to the number of sites in the Hilbert space.
+                This is equivalent to subsampling the Markov chain.
+        reset_chains: If True, resets the chain state when `reset` is called on every new sampling (default = False).
+        machine_pow: The power to which the machine should be exponentiated to generate the pdf (default = 2).
+        dtype: The dtype of the states sampled (default = np.float64).
     """
-    if not isinstance(hilbert, AbstractParticle):
+    if not isinstance(hilbert, ContinuousHilbert):
         raise ValueError("This sampler only works for Continuous Hilbert spaces.")
 
     from .rules import GaussianRule
