@@ -1,3 +1,16 @@
+# Copyright 2021 The NetKet Authors - All rights reserved.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 import jax
 import jax.numpy as jnp
 
@@ -24,7 +37,7 @@ class GaussianRule(MetropolisRule):
         pbc = jnp.array(hilb.n_particles * hilb.pbc)
         boundary = jnp.tile(pbc, (n_chains, 1))
 
-        Ls = jnp.array(hilb.n_particles * hilb.extend)
+        Ls = jnp.array(hilb.n_particles * hilb.extent)
         modulus = jnp.where(jnp.equal(pbc, False), jnp.inf, Ls)
 
         prop = jax.random.normal(key, shape=(n_chains, hilb.size)) * rule.sigma
